// creature.cpp
// by Glenn G. Chappell
// September 2003
//
// For CS 381
// GLUT Menu Demo
//
// Draws a creature face, with attributes settable using a GLUT pop-up menu

#include <iostream>
using std::cerr;
using std::endl;
#include <stdlib.h>  // Some versions of MS-Vis C++ have broken <cstdlib>
//using std::exit;
#include <GL/glut.h> // GLUT stuff, includes OpenGL headers as well


// Global variables
// Window/viewport
const int startwinsize = 400;     // Starting window width & height, in pixels

// Creature stuff
bool blueeyes = false;            // True: eyes are blue, false: brown
bool mouthmoving = false;         // True: mouth is moving, false: not moving

const double mouthsizemax = 0.12; // Max & min for moutsize (below)
const double mouthsizemin = 0.01;
double mouthsize = mouthsizemin;  // Distance from centerline to top of mouth
const double mouthspeed = 0.01;   // Speed of mouth movement
double currentmouthspeed = mouthspeed;
                                  // Amount mouthsize currently changes from frame to frame


// display
// The GLUT display function
void display()
{
   glClear(GL_COLOR_BUFFER_BIT);

   // Head
   glColor3d(0.9, 0.3, 0.1);  // Reddish-orange #1
   glBegin(GL_QUADS);
      glVertex2d(0.2, 0.2);
      glVertex2d(0.8, 0.2);
      glVertex2d(0.8, 0.8);
      glVertex2d(0.2, 0.8);
   glEnd();

   // Ears
   glColor3d(0.9, 0.4, 0.1);  // Reddish-orange #2
   glBegin(GL_QUADS);
      // Right ear
      glVertex2d(0.2, 0.4);
      glVertex2d(0.2, 0.6);
      glVertex2d(0.1, 0.65);
      glVertex2d(0.1, 0.35);
      // Left ear
      glVertex2d(0.8, 0.4);
      glVertex2d(0.8, 0.6);
      glVertex2d(0.9, 0.65);
      glVertex2d(0.9, 0.35);
   glEnd();

   // Eyes
   glBegin(GL_QUADS);
      // Eyeballs ("eyesquares"??)
      glColor3d(1.0, 1.0, 1.0);  // White
      // Right eyeball
      glVertex2d(0.25, 0.55);
      glVertex2d(0.25, 0.75);
      glVertex2d(0.45, 0.75);
      glVertex2d(0.45, 0.55);
      // Left eyeball
      glVertex2d(0.55, 0.55);
      glVertex2d(0.55, 0.75);
      glVertex2d(0.75, 0.75);
      glVertex2d(0.75, 0.55);

      // Irises
      if (blueeyes)
         glColor3d(0.1, 0.5, 0.9);  // Blue
      else
         glColor3d(0.6, 0.4, 0.2);  // Brown
      // Right iris
      glVertex2d(0.3, 0.6);
      glVertex2d(0.3, 0.7);
      glVertex2d(0.4, 0.7);
      glVertex2d(0.4, 0.6);
      // Left iris
      glVertex2d(0.6, 0.6);
      glVertex2d(0.6, 0.7);
      glVertex2d(0.7, 0.7);
      glVertex2d(0.7, 0.6);

      // Pupils
      glColor3d(0.0, 0.0, 0.0);  // Black
      // Right pupil
      glVertex2d(0.33, 0.63);
      glVertex2d(0.33, 0.67);
      glVertex2d(0.37, 0.67);
      glVertex2d(0.37, 0.63);
      // Left pupil
      glVertex2d(0.63, 0.63);
      glVertex2d(0.63, 0.67);
      glVertex2d(0.67, 0.67);
      glVertex2d(0.67, 0.63);
   glEnd();

   // Mouth
   glColor3d(0.2, 0.0, 0.0);  // Dark red
   glBegin(GL_QUADS);
      glVertex2d(0.25, 0.35);
      glVertex2d(0.5,  0.35+mouthsize);
      glVertex2d(0.75, 0.35);
      glVertex2d(0.5,  0.35-mouthsize);
   glEnd();

   // Legs & feet
   glColor3d(0.9, 0.4, 0.1);  // Reddish-orange #2
   glBegin(GL_QUADS);
      // Right leg
      glVertex2d(0.4,  0.2);
      glVertex2d(0.45, 0.2);
      glVertex2d(0.45, 0.1);
      glVertex2d(0.4,  0.1);
      // Right foot
      glVertex2d(0.45, 0.1);
      glVertex2d(0.3,  0.1);
      glVertex2d(0.3,  0.05);
      glVertex2d(0.45, 0.05);
      // Left leg
      glVertex2d(0.6,  0.2);
      glVertex2d(0.55, 0.2);
      glVertex2d(0.55, 0.1);
      glVertex2d(0.6,  0.1);
      // Left foot
      glVertex2d(0.55, 0.1);
      glVertex2d(0.7,  0.1);
      glVertex2d(0.7,  0.05);
      glVertex2d(0.55, 0.05);
   glEnd();

   glutSwapBuffers();
}


// idle
// The GLUT idle function
void idle()
{
   // Print OpenGL errors, if there are any (for debugging)
   if (GLenum err = glGetError())
   {
      cerr << "OpenGL ERROR: " << gluErrorString(err) << endl;
   }

   // Mouth animation
   if (mouthmoving)
   {
      mouthsize += currentmouthspeed;  // Move it
      if (mouthsize > mouthsizemax)    // Hit upper limit?
      {
         mouthsize = mouthsizemax;
         currentmouthspeed = -mouthspeed;
      }
      if (mouthsize < mouthsizemin)    // Hit lower limit?
      {
         mouthsize = mouthsizemin;
         currentmouthspeed = mouthspeed;
      }
      glutPostRedisplay();             // Draw it
   }
}

// reshape
// The GLUT reshape function
void reshape(int w, int h)
{
   // Set up viewport
   glViewport(0, 0, w, h);

   // Set up the projection
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (w > h)
      gluOrtho2D(0.5-0.5*w/h, 0.5+0.5*w/h, 0.0, 1.0);
   else // w <= h
      gluOrtho2D(0.0, 1.0, 0.5-0.5*h/w, 0.5+0.5*h/w);

   glMatrixMode(GL_MODELVIEW);  // Always go back to model/view mode
}


// handle_main_menu
// Menu handling callback function for our menu
void handle_main_menu(int value)
{
   switch (value)
   {
   case 1:   // Toggle eye color
      blueeyes = !blueeyes;
      glutPostRedisplay();
      break;
   case 2:   // Toggle mouth motion
      mouthmoving = !mouthmoving;
      glutPostRedisplay();
      break;
   case 3:   // Quit
      exit(0);
      break;
   case 999:  // Other menu items do nothing
      break;
   }
}


// make_main_menu
// Creates the main menu,
//  using function handle_main_menu as the handler callback
void make_main_menu()
{
   glutCreateMenu(handle_main_menu);
   glutAddMenuEntry("Toggle eye color", 1);
   glutAddMenuEntry("Toggle mouth motion", 2);
   glutAddMenuEntry("Quit", 3);
   glutAddMenuEntry("------------------------", 999);
   glutAddMenuEntry("CREATURE - GLUT Menu Demo", 999);
   glutAddMenuEntry("by Glenn G. Chappell", 999);
   glutAddMenuEntry("for CS 381, fall 2003", 999);
   glutAttachMenu(GLUT_RIGHT_BUTTON);
}


// init
// Initializes GL states
// Called by main after window creation
void init()
{
   make_main_menu();

   glClearColor(1.0, 1.0, 1.0, 0.0);
}


int main(int argc, char ** argv)
{
   // Initialize OpenGL/GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);

   // Make a window
   glutInitWindowSize(startwinsize, startwinsize);
   glutInitWindowPosition(50, 50);
   glutCreateWindow("Creature - right mouse for menu");

   // Initialize GL states & register callbacks
   init();
   glutDisplayFunc(display); 
   glutIdleFunc(idle);
   glutReshapeFunc(reshape);

   // Do something
   glutMainLoop();

   return 0;
}
